PRO Rivwidth_v03_r2, fname

;Copyright (c) 2002-2012, Tamlin Pavelsky
;All rights reserved.
;
;Redistribution and use in source and binary forms, with or without
;modification, are permitted provided that the following conditions are met:
;     * Redistributions of source code must retain the above copyright
;       notice, this list of conditions and the following disclaimer.
;     * Redistributions in binary form must reproduce the above copyright
;       notice, this list of conditions and the following disclaimer in the
;       documentation and/or other materials provided with the distribution.
;     * The name of Tamlin Pavelsky may not be used to endorse or promote products
;       derived from this software without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY TAMLIN PAVELSKY ``AS IS'' AND ANY
; EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
; DISCLAIMED. IN NO EVENT SHALL TAMLIN PAVELSKY BE LIABLE FOR ANY
; DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
; (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
; ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
; SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


;Original Author:  Tamlin Pavelsky, University of North Carolina, Chapel Hill
;Contributing Authors:  Larry Smith, UCLA; Doug Alsdorf, Ohio State University; Michael Durand, Ohio State University
;Original Date:  11/11/2002  Revised:  10/29/2012
;Minor Bug fixes by D.Mayer, Clark University: 03/2011

;1) Preliminaries
;1.1) Get input data: this controls program flow
rundata=read_inputs(fname) ;This is now the only line in code that needs to be modified
tm=systime(1)

;1.2) Define width, height, resolution, and whether you're working on a braided or unbraided river from the rundata structure
width=rundata.w
height=rundata.h
resolution=rundata.r
braidflag=rundata.b

;1.3) Develop a channel mask for the desired river system.  This is only done once, at the beginning.  If, in the paramter file, calc_masks is set to 0, then we will simply open the existing mask files.
if(rundata.s1 eq 1) then begin 
  ;1.3.1) Reads in a classified binary water mask mask: this mask can contain water bodies outside of the desired river, but these bodies must be separated from the river by at least one pixel.
  chan_mask=read_binary(rundata.f0,DATA_DIMS=[width,height],DATA_TYPE=1)
  if max(chan_mask) gt 1 then chan_mask(where(chan_mask gt 1))=1  
  if braidflag ne 1 then label=label_region(chan_mask, /ULONG) else label=label_region(chan_mask, /all_neighbors, /ULONG) ;use all_neighbors for braiding, remove it for simple channel rivers.
  ;1.3.2) Removes all water bodies except for the largest one.  If there is a body of water in the image of greater extent than the desired river network, it will have to be removed manually prior to this stage.
  labelhist=histogram(label, binsize=1, /L64)
  lhlength=size(labelhist, /dimensions, /L64)
  maxhist=where(labelhist eq max(labelhist[1:lhlength[0]-1]))
  ;By default, this next line finds the largest contiguous water area in your mask. You can also comment in the line below and comment this one out to specify
  ;the feature in the water mask for which you want to calculate widths.  Simply change the x,y values and add or substract individual locations.  You just have
  ;to specify one pixel location that falls within the feature you want to focus on.  Note that you also need to make the same change below, on lines 86-87
  eraser=where(label ne maxhist[0])  
  ;eraser=where(label ne label[2901,1987] and label[*,*] ne label[437,6703] and label[*,*] ne label[16142,4792])
  chan_maskr=chan_mask
  if eraser ne [-1] then chan_mask(eraser)=0
  
  ;1.3.3) Outputs the final channel mask.  If the original channel mask is already clean (ie like the input for prior version of RivWidth), then it will be identical to the input water mask.
  openw, 1, rundata.f2
  writeu, 1, chan_mask
  close, 1
  
  print, "Channel mask successfully calculated in (seconds):  ", systime(1)-tm

  ;1.3.4) Optional section that exports the original water mask with boundary pixels highlighted around all water bodies.
  ;imgbnd=chan_maskr
  ;operator=[[1,1,1],[1,0,1],[1,1,1]]
  ;imgbnd(where(chan_maskr eq 0 and convol(chan_maskr,operator) gt 0))=2
  ;
  ;openw, 2, "/Volumes/Data/files/SWOT/Paris/Ohio/Ohio_bound_image"
  ;writeu, 2, imgbnd
  ;close, 2
  
  ;1.4)  Removes all islands from within the mask and uses a series of erosions and dilations to develop the river mask, from which the river center line will be computed
  operator=[[0b,1,0],[1,0,1],[0,1,0]] ;this is a 4-connect operator that is used for dilations and erosions
  ;1.4.1)  Begins with the original water mask and removes all water bodies except the largest, which is presumed to be the river/river network of interest.  
  ;if braidflag eq 1 then for x=0,0 do chan_maskr=dilate(chan_maskr, operator) else chan_maskr=chan_mask  ;If we are in a braided system, then we perform 2 dilations before removing islands to avoid removing key channel pixels that may be disconnected from the main river
  label=label_region(chan_maskr, /all_neighbors, /ulong)
  labelhist=histogram(label, binsize=1, /L64)
  lhlength=size(labelhist, /dimensions)
  maxhist=where(labelhist eq max(labelhist[1:lhlength[0]-1]))
  eraser=where(label ne maxhist[0])
  ;eraser=where(label ne label[2901,1987] and label[*,*] ne label[437,6703] and label[*,*] ne label[16142,4792])  
  if eraser ne [-1] then chan_maskr(eraser)=0
  
  ;flipim is a version of the binary mask in which water=0 and land=1.  It is used to label and remove islands in the river.
  if where(chan_maskr ne 0) ne [-1] then flipim=abs(chan_maskr-1) else print, "big error"
  labelflip=uint(label_region(flipim, /ulong))

  label=0
  chan_maskr=0
  p=1
  dcnt=1
  
  ;1.4.2)  Loop that first detects all land areas that intersect the edge of the image.  These are retained as "land" in the river mask.
  ;All other land polygons are considered to be islands in the river and their values are changed accordingly.  With each loop iteration,
  ;we dilate the river area using a 4-connect operator in order to fully bound islands that are incompletely bounded by water in the original 
  ;channel mask.  At the moment, this process ony happens once, but this number is arbitrary and can be modified in the last line of the loop (dcnt gt x), below.  
 
  while p eq 1 do begin 
    labelhist=histogram(labelflip, binsize=1, /L64)
    lhlength=size(labelhist, /dimensions)
  
    side1hist=histogram(labelflip[1,0:height-1], binsize=1) 
    s1s=size(side1hist, /dimensions)
    side2hist=histogram(labelflip[width-2,0:height-1], binsize=1)
    s2s=size(side2hist, /dimensions)
    side3hist=histogram(labelflip[0:width-1,1], binsize=1)
    s3s=size(side3hist, /dimensions)
    side4hist=histogram(labelflip[0:width-1,height-2], binsize=1)
    s4s=size(side4hist, /dimensions)
    
    sidespoly=uint([where(side1hist ne 0),where(side2hist ne 0),where(side3hist ne 0),where(side4hist ne 0)])
    sizepoly=size(sidespoly, /dimensions)

    for y=0, sizepoly[0]-1 do begin
      if where(labelflip eq sidespoly[y]) ne [-1L] and sidespoly[y] ne 0 then begin
        labelflip(where(labelflip eq sidespoly[y]))=65535
      endif 
    endfor

    if where(labelflip gt 0) ne [-1] then begin   
      labelflip(where(labelflip gt 0 and labelflip ne 65535))=0
      labelflip(where(labelflip eq 65535))=1
    
      riv_maskfl=labelflip
      riv_mask=byte(abs(labelflip-1))
    
      riv_maskd=dilate(riv_mask,operator)
      flipim=abs(riv_maskd-1)
      labelflip=uint(label_region(flipim, /ulong))
      dcnt++
    endif else p=0

    if dcnt gt 0 then p=0  ;Modify the numerical value in "dcnt gt x" to do either more or fewer dilations
  endwhile
  
  ;OLD CODE NOT CURRENTLY USED.  This looks like just another dilation, but in fact we are dilating the opposite direction (ie eroding "river" areas) because we are applying the dilations to a 
  ;flipped version of the image where water=0 and land=1.  We do as many "erosions" as we have done dilations prior to this point.  
  ;if dcnt ge 2 then begin
    ;for q=0,dcnt-1 do riv_maskfl=dilate(riv_maskfl,operator)  
  ;endif else print, "Error in computing river mask.  Make sure your input image assigns values of 1 to water areas, not 255."
  
  ;Rivmask is the final river mask to be used in calculating the river center line.
  riv_mask=byte(abs(riv_maskfl-1))

  ;1.5) Outputs the final river mask and final channel mask to file locations specified in the input parameter file.
  openw, 2, rundata.f1
  writeu, 2, riv_mask
  close, 2
  
  print, "River mask successfully calculated in (seconds): ", systime(1)-tm
endif else begin

  ;If the river mask and channel mask have already been created, we can skip the steps above and simply open the two images, which we do here.

  riv_mask=read_binary(rundata.f1,DATA_DIMS=[width,height],DATA_TYPE=1)

  chan_mask=read_binary(rundata.f2,DATA_DIMS=[width,height],DATA_TYPE=1)

endelse

;2) Calculate the initial centerline from the River Mask
if rundata.s2 eq 1 then begin  
  Cline=CL_Calc(riv_mask, width, height, tm)
  
  ;2.1) Outputs the centerline image to the location specified in the parameter file
  openw, 3, rundata.f4
  writeu, 3, Cline
  close, 3

  print, "Initial centerline successfully calculated in (seconds): ", systime(1)-tm 
  print, "You may now need to modify the parameter file to include correct start point and end point coordinates for the desired river center line segment"
  
endif

;;;;;;;;;;;;;;;;;;;;;;;;;;
;This next section controls the remainder of RivWidth.  It takes in the initial centerline and the channel mask, extracts a one-pixel, directional centerline based on 
;manually-input start and end points (which must fall on the initial centerline), determines the line segment orthogonal to the centerline at each pixel, and calculates
;the total flow width along that centerline.
;;;;;;;;;;;;;;;;;;;;;;;;;;

if(rundata.s3 eq 1) then begin
  spx=rundata.spx
  spy=rundata.spy
  epx=rundata.epx
  epy=rundata.epy
  agval=1 ;set this value to the number of pixels that you want to aggregate.  In other words, if you want mean widths over a length of 10 centerline pixels, then this value should be 10.

  ;This calculation of a distance image is no longer used for centerline calculations--instead, it's just used to determine the length of the orthogonal for each channel cross-section.
  dist_im=morph_distance(riv_mask, neighbor_sampling=3) 
  
  ;openw,2,rundata.f3
  ;writeu, 2, dist_im  ;outputs distance image for future analysis
  ;close, 2

  ;3) Calculate the final centerline
  ;3.1) If the initial centerline was calculated previously, import it into IDL
  if(rundata.s2 eq 0) then begin
    Cline=read_binary(rundata.f4,DATA_DIMS=[width,height],DATA_TYPE=1)    
  endif

  ;3.2) Calculate the final centerline.  dynamic_vector_func computes a 2-dimensional array that represents the shortest path along the intial centerline
  ;from the start point (spx,spy) to the end point (epx,epy).  It returns this in vector form (the coordinates) and also saves it in image form.
  vector_out=dynamic_vector_v3(Cline,width,height,spx,spy,epx,epy)
  print, "Finished calculating final centerline in (seconds): ", systime(1)-tm

  ;3.3) Compute the line segment (or, more apprpriately the endpoints of the line segment) orthogonal to the final centerline at each centerline pixel.
  ;Note that it is possible you might want to modify the values "dist" and "seglen" in the function return_orthogonal_v3.
  orth_array=return_orthogonal_v3(vector_out, width, height, agval, dist_im)
  print, "Finished calculating orthogonals to centerline in (seconds): ", systime(1)-tm

  ;3.4) Zoutput determines the total flow width along the orthogonal segment for each centerline pixel.  
  zoutput=z_values_v3(chan_mask, orth_array, agval, resolution, riv_mask)

  openw, 3, rundata.f7  ;opens up a text file to write the final width values into
  printf, 3, "xcoord, ycoord, width, num_channels"  ;prints column headings for the final .csv width output file
  openw, 5, rundata.f6  ;opens up an image file to store the final width image
  
  zoutput_size=size(zoutput, /Dimensions)
  zlength=zoutput_size[0]

  out_image=fltarr(width,height)
  agcounter=0
  agaverage=0
  agdist=0
  startpoint=intarr(2)

  ;3.5) The following code computes average widths for defined river segment length.  The default length is 1 pixel (ie compute the width at every pixel), but the value of "agval" (Line 192) can be modified to any value
  ;up to the total centerline length.
 if agval gt 1 then begin
  for f=0,zlength-1 do begin
    if zoutput[f,4] eq 0 and zoutput[f,5] eq 0 AND f lt zlength-1 then begin
    startpoint=[zoutput[f+1,4],zoutput[f+1,5]]
    endif
    if zoutput[f,4] gt 0 and zoutput[f,5] gt 0 then begin
      agdist=distance(startpoint[0],startpoint[1],zoutput[f,4],zoutput[f,5])
      if fix(agdist) le agval then begin
        agaverage=agaverage+zoutput[f,6]
        agcounter=agcounter+1
      endif
      if fix(agdist) ge agval then begin
        out_image(zoutput[(f-(fix(agcounter)-1)/2),4],zoutput[(f-(fix(agcounter)-1)/2),5])=agaverage/agcounter
        printf,3,zoutput[(f-(fix(agcounter)-1)/2),4],zoutput[(f-(fix(agcounter)-1)/2),5],agaverage/agcounter,zoutput[f,7],format='((I6),",",(I6),",",(I6),",",(I6))'
        agdist=0
        agcounter=0
        agaverage=0
        startpoint=[zoutput[f,4],zoutput[f,5]]
      endif   
    endif
  endfor
 endif else begin
    ;This code gets run if we are computing the width for every centerline pixel and not doing any aggregating.  The code below simply outputs the final data to a text file and assigns the appropriate width
    ;values to the output width image (out_image).
    for f=0,zlength-1 do begin
      if zoutput[f,4] ne 0 or zoutput[f,5] ne 0 then printf,3,zoutput[f,4],zoutput[f,5],zoutput[f,6],zoutput[f,7],format='((I6),",",(I6),",",(I6),",",(I6))'
      out_image[zoutput[f,4],zoutput[f,5]]=zoutput[f,6]
    endfor
 endelse
    
  writeu, 5, fix(out_image)  ;outputs the final width image
  
  print, "Finished calculating river width in (seconds): ", systime(1)-tm

  close, /all
endif

END

;;;;;;;;;;;;;;;;;;;
;The following functions are called by the control function above.  Each performs one task in RivWidth:
;CL_Calc:  Takes in the River Mask and calculates an initial centerline
;Dynamic_Vector_v3:  Takes in the initial centerline and user-defined start and endpoints and outputs a one-pixel, directional (start to end) centerline vector.
;Return_Orthogonal_v3:  Takes in the final centerline and determines the line segment orthogonal to the centerline at each centerline pixel
;z_values_v3:  Takes in the orthogonal endpoints and the centerline and calculates the total flow width along each orthogonal
;
;There are also a variety of minor supporting functions below, including eight_vector, getpoints, distance, and bound.  The function read_inputs takes in the parameter file and parses it to provide
;input to the control function above.
;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;
;CL_Calc is a function, called above, that calculates the initial centerline for the river channel.  This is based on edge-detection methods detailed in Pavelsky and Smith (2008, IEEE GRSL).
;;;;;;;;;;;;;;;;;;

function CL_Calc, img, width, height, tm

;1) Create an image (imgbnd) where the nonriver pixels that are adjacent to river pixels have a value of 2, and all other pixels have a value of 0.
operator=[[1,1,1],[1,0,1],[1,1,1]]
imgbnd=img
imgbnd(where(img eq 0 and convol(img,operator) gt 0))=2
imgbnd(where(imgbnd lt 2))=0

;2) Calculate an initial, noneuclidean distance map using the IDL function morph_distance
MD=morph_distance(img,neighbor_sampling=3)

;3) Calculate some initial inputs for the loop to follow that will refine MD into a true euclidean distance map
pix=long(where(MD gt 2))
maxpix=size(pix, /dimensions)
tst=0
x=0l

;4) Calculates a generic, square euclidean distance map that shows the distance from the image center
MDm=fix(max(MD))+3
distmap=shift(dist(MDm*2),fix(MDm),fix(MDm))

;5) The following loop goes through each river pixel in the image.  For each pixel, it subsets the generic euclidean distance map to be slightly larger than the noneuclidean distance map value.
;   This distance map subset is centered on top of the river pixel in question.  Then, the minimum distance from the set of boundary pixels (in imgbnd) that fall within the distance map window is
;   determined and assigned to the river pixel under consideration.  This provides the correct, euclidean distance without the necessity of searching through many individual pixel distances.
while tst eq 0 do begin
  MDv=fix(MD[pix[x]])+2
  distmap2=distmap[MDm-MDv:MDm+MDv,MDm-MDv:MDm+MDv]  
  pixx=pix[x] mod width
  pixy=pix[x]/width
  if MDv le pixx and MDv le pixy and pixx+MDv lt width and pixy+MDv lt height then begin
    flipsqr=imgbnd[pixx-MDv:pixx+MDv,pixy-MDv:pixy+MDv]
    if max(flipsqr) lt 2 then print, "error"
    distmap2(where(flipsqr ne 2))=0
  endif
  MD[pixx,pixy]=min(distmap2(where(distmap2 ne 0))) 
  
  if x eq maxpix[0]-1 then tst=1 
  x++ 
endwhile

;6) With the true euclidean distance map computed, we can now calculate the centerline.  

;6.1) The horizontal and vertical laplacian operators
operator_hor=[-0.5,0,0.5]
operator_vert=[[0.5],[0],[-0.5]]

;6.2) The two operators above are convolved with the distance map to produce a laplacian map.  River values will be close to 1 except near the river centerline, where they will be near 0.
MD_Deriv=convol(MD, operator_hor)^2 + convol(MD, operator_vert)^2
;print, "MD_Deriv ", MD_Deriv[5088,19565], MD_Deriv[9436,26991]
MD=0

;6.3) The following steps extract the centerline from the raw laplacian image and refine it by removing any centerline pixels not attached to the main centerline.  Note that you can change the
;dividing value of MD_Deriv (currently 0.92) to any value you wish.  The higher it is, the more pixels will be classified as "centerline" pixels.  The lower it is, the fewer pixels will be included.
;Note that if you lower it too much, then you're liable to get a centerline with many gaps, which is not ideal.

;6.3.1) Extracts the initial raw centerline.
MD_Deriv(where(MD_Deriv gt 0.92 or img eq 0))=2
MD_Deriv(where(MD_Deriv le 0.92))=1.
MD_Deriv(where(MD_Deriv eq 2))=0

;6.3.2) Labels each contiguous region of centerline pixels.
Cline_region=label_region(MD_Deriv, /all_neighbors, /ulong)

;6.3.3) Removes small centerline regions that clutter up the image (anything smaller than 100 pixels).  Nearly always, the true centerline will be
;longer than 100 pixels.
labelhist=histogram(Cline_region, binsize=1)
lhlength=size(labelhist, /dimensions)
maxhist=where(labelhist gt 100 and labelhist ne 0, count) 
for x=0,count-1 do Cline_region(where(Cline_region eq maxhist[x] and img ne 0))=1
Cline_region(where(Cline_region gt 1))=0
Cline_region(where(Cline_region eq 1))=1
Cline_region=byte(Cline_region*MD_Deriv)

Rough_cl=rivmask_linker(Cline_region,width,height)

;7) Returns the finished centerline
return, Rough_cl

end

function rivmask_linker, w_mask, w, h
  ;This function added in Summer 2012 connects individual centerline segments when, due to strange geometries that come up very occoasionally,
  ;there is a 1-pixel gap that appears in the initial centerline.  It solves the very annoying problem of having to find these minute gaps manually.
  w_bnd=bytarr(w,h)
  operator=[[1,1,1],[1,0,1],[1,1,1]]
  w_bnd(where(w_mask eq 0 and convol(w_mask,operator) gt 0))=1
  
  label=label_region(w_mask, /all_neighbors)
  
  fix_pts=reg_test(w,h,w_bnd,label)
  
  w_mask(where(fix_pts ne 0))=1

  return, w_mask
  
end

function reg_test, w, h, w_bnd, label
  ;This function is a subfunction of rivmask_linker, above.  It identifies those pixels where there are two or more adjacent centerline pixels
  ;that fall within different centerline regions. In other words, it finds the pixels that link up different centerline regions, thus solving
  ;the annoying problem identified above.

  w_bnd_1=where(w_bnd eq 1)
  w_bnd_size=size(where(w_bnd eq 1),/dimensions)
  w_bnd_len=w_bnd_size[0]

  output=w_bnd-w_bnd 

  county=0

  for m=0,w_bnd_len-1 do begin
 
    if w_bnd_1(m) mod w gt 1 and w_bnd_1(m)/w gt 1 and w_bnd_1(m) mod w lt w-2 and w_bnd_1(m)/w lt h-2 then begin
      fix_hist=histogram(label((w_bnd_1(m) mod w)-1:(w_bnd_1(m) mod w)+1, (w_bnd_1(m)/w)-1:(w_bnd_1(m)/w)+1)) 
      endif else begin
        fix_hist=bytarr(3,3)
        endelse
        if size(where(fix_hist gt 0), /dimensions) gt 2 then begin
        output(w_bnd_1(m))=1
        county++
        endif

  endfor

  return, output

end



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Begin Dynamic_Vector Code:  Dynamic vector uses a minimum cost search algorithm to determine the shortest path from
;the start point (spx,spy) to the end point (epx,epy) contained within the initial centerline.  
;This should not be edited except in very unusual circumstances
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function dynamic_vector_v3, input_image, width, height, spx, spy, epx, epy

;I can't remember why I have to do this, but it turns out right if I do and doesn't work if I don't.  That's the problem with writing
;code over the course of 8 years...
epx=epx-1
epy=epy-1

;1) Variables and arrays initialized that will be used in the loop below:
count=0L ;initializes number of values in queue
cost=0L ;initializes the cost value
signal=0 ;a signal variable that is switched to 1 when the destination pixel is reached
step=bytarr(width,height) ;an array that stores all centerline pixels that have already been searched
cim=lonarr(width,height,2) ;an array thot holds the x/y coordinates of the adjacent centerline pixel from which the pixel in question was reached.
out_image=bytarr(width,height) ;the image that will contain the final, 1-pixel centerline
queue=lonarr(height*2,5) ;the queue that contains the pixels to be searched for (epx,epy) 

;2) Initialize the queueu by adding the start pixel
queue[count,0:4]=[spx,spy,cost,0,0]  ;initializes the first pixel in the queue with the start pixel
step[spx,spy]=1
count=count+1

;cur=[0,0,0]  

;3) Each iteration of the following loop extracts a pixel from the queue and determines if it is the end pixel (epx, epy)
;   If it is, then we can output our final centerline.  If not, then we find all of the pixels in an eight-connect neighborhood
;   that have not already been added to the queue and add them to the queue.  

while signal ne 1 AND count ne 0 do begin
  
    cur=reform(queue[0,0:4]) ;extracts the first pixel in the queue)
    cim[cur[0],cur[1],0]=cur[3]  ;keeps track of the prior pixel coordinates so we can determine our path back to (spx,spy)
    cim[cur[0],cur[1],1]=cur[4]
    queue[0:count-1,0:4]=queue[1:count,0:4] ;resets the queue by removing the pixel we are about to consider and shifting all other pixels up one slot   
    count=count-1 ;keeps track of the queue length
    ;if count le 0 then print, "Error in centerline.  Check for gaps at ", cur[0], cur[1]
    if cur[0] eq epx AND cur[1] eq epy then begin  ;If we have reached our end pixel (epx,epy), then this runs
       signal=1  ;sets signal variable to stop our while loop
       out_image[cur[0],cur[1]]=1  ;assigns a value of 1 to our end pixel (epx,epy) in the output image
       out_array=lonarr(cur[2],2)  ;initializes our output array, which just contains the x,y coordinates of the centerline pixels
       larray=[cur[3],cur[4]]  ;initializes larray, which allows us to go back through our search and determine the fastest route back to (spx,spy) from (epx,epy)
       for x=0,cur[2]-1 do begin
         out_array[cur[2]-x-1,0]=larray[0]
         out_array[cur[2]-x-1,1]=larray[1]
         out_image[larray[0],larray[1]]=1
         larray=[cim[larray[0],larray[1],0],cim[larray[0],larray[1],1]]
       endfor
    endif else begin  ;If we have not yet reached our end pixel (epx,epy), then this runs
       ec=eight_vector(cur[0],cur[1],input_image, width, height)  ;determines the number of initial centerline pixels in an 8-connect neighborhood around our current pixel
       if ec ge 1 AND cur[0] ne 0 AND cur[1] ne 0 AND cur[0] ne width-1 ANd cur[1] ne height-1 then begin
         neigh=getpoints(input_image,ec,cur[0],cur[1], cur[2]) ;gets the coordinates of all pixels in an 8-connect neighborhood with values of 1
         for x=0,ec-1 do begin
          if step[neigh[x,0],neigh[x,1]] eq 0 then begin  ;if it hasn't already been added, add each pixel obtained using getpoints to the queue
              queue[count,0:4]=[neigh[x,0],neigh[x,1],neigh[x,2],neigh[x,3],neigh[x,4]]              
              step[neigh[x,0],neigh[x,1]]=1
              ;print, queue[0,0], queue[0,1], queue[0,2], queue[0,3], queue[0,4]
              count=count+1
          endif
         endfor
       endif 
    endelse
endwhile

;;this outputs an image showing the final one pixel wide center line
;openw, 2, FILEPATH('binary_line',ROOT_DIR='e:\MODIS\Lena\Karen\gina\')
;
;writeu, 2, out_image
;
;close, 2

;4) Return the final centerline array
return, out_array

end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;subfunctions of function dynamic_vector_v3 above
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


function eight_vector, spx, spy, in, width, height

  if spx eq 0 OR spx eq width-1 OR spy eq 0 or SPY eq height-1 then begin
      return, fix(3)
  endif else begin
      eight_conn=in[spx,spy+1]+in[spx,spy-1]+in[spx-1,spy]+in[spx+1,spy]+in[spx-1,spy+1]+in[spx+1,spy+1]+in[spx+1,spy-1]+in[spx-1,spy-1]
      return, eight_conn
  endelse

end

function getpoints, in, ec, cx, cy, cost

    outpoints=lonarr(ec+1,5)
    n=0

    if in[cx+1,cy] eq 1 then begin
      outpoints[n,0:4]=[cx+1,cy,cost+1,cx,cy]
      n++
    endif
    if in[cx-1,cy] eq 1 then begin
      outpoints[n,0:4]=[cx-1,cy,cost+1,cx,cy]
      n++
    endif
    if in[cx,cy+1] eq 1 then begin 
      outpoints[n,0:4]=[cx,cy+1,cost+1,cx,cy]
      n++
    endif
    if in[cx,cy-1] eq 1 then begin
      outpoints[n,0:4]=[cx,cy-1,cost+1,cx,cy] 
      n++
    endif
    if in[cx+1,cy-1] eq 1 then begin 
      outpoints[n,0:4]=[cx+1,cy-1,cost+1,cx,cy]
      n++
    endif
    if in[cx+1,cy+1] eq 1 then begin 
      outpoints[n,0:4]=[cx+1,cy+1,cost+1,cx,cy]
      n++
    endif
    if in[cx-1,cy-1] eq 1 then begin
      outpoints[n,0:4]=[cx-1,cy-1,cost+1,cx,cy]
      n++
    endif
    if in[cx-1,cy+1] eq 1 then begin
      outpoints[n,0:4]=[cx-1,cy+1,cost+1,cx,cy]
      n++
    endif

    return, outpoints

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Begin Return Orthogonal Code
;The variabls seglen and dist may be edited.  Everything else should be left as is except under unusual circumstances
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function return_orthogonal_v3, input, width, height, agval, distim

;1) Initialize variables to be used further down in the function

orth_image=bytarr(width,height) ;intializes an image that contains the centerline pixel and the corresponding orthogonal endpoint pixels

seglen=15L  ;This is the length of the centerline segment against which the orthogonal is be computed.  It should probably be equal to approximately the river width. Rivwidth is sensitive to this number
            ;only in rivers that are highly sinuous.  Also, see lines 592-596 below.  When working in rivers narrower than seglen, RivWidth will automatically adjust seglen down to reduce potential error
            ;The impact of seglen on width calculations was investigated extensively by Zach Miller in Fall 2012 in the Mississippi Basin.

seglenh=long((seglen-1)/2)     ;segment length / 2
n=long(seglenh)
input_size=size(input, /Dimensions)
cline_length=input_size[0]
ortharr=lonarr(cline_length,8) ;the final output array listing each centerline pixel coordinates and the coordinates of its two orthogonal endpoints
;distfull=shift(dist(1000,1000),500,500) ;This is a generic euclidean distance map showing distance from the image center.  Note that it must be smaller than the image under consideration.
if width gt 3000 and height gt 3000 then begin
  dfval=3000
  distfull=shift(dist(3000,3000),1500,1500) ;This is a generic euclidean distance map showing distance from the image center.  Note that it must be smaller than the image under consideration.
endif else begin  ;If the image is smaller than the dimensions above, then the smaller dimension of the input image is used to calculate the generic distance map.
  dfval=min([width,height])
  distfull=shift(dist(dfval,dfval),dfval/2,dfval/2)
endelse

;2) The following code goes through each pixel in the centerline and calculates the coordinates of the orthogonal endpoints using simple trigonometry.
;   Since we know the length of the centerline segment against which we are calculating the orthogonal and we know the length of the orthogonal, we can
;   Compute the length of the hypotenuse formed by these two segments.  We can then find the two points (one on each side of the river) that have the
;   Correct distances from both the centerline pixel and an end of the centerline segment against which the orthogonal will be computed.

while n lt cline_length-seglenh do begin
  
    ;The following section of code adjusts seglen and seglenh downwards if width calculations are being performed in narrow rivers.  A threshold of 11 pixels for seglen was chosen
    ;based on direct experimentation across a wide range of rivers in the Mississippi Basin.  It can be modified as necessary, however.
    maxwval=fix(fix(max(distim[input[n,0]-seglenh:input[n,0]+seglenh,input[n,1]-seglenh:input[n,1]+seglenh])*2))
    if maxwval lt seglen then begin 
      seglen=long(maxwval) & seglenh=long(maxwval/2)
      if seglen lt 11l then seglen=11l & seglenh=5l
    endif

    ;computes the length of the orthogonal based on the maximum distance value from a window of  the river mask distance map including the centerline pixel
    ; +/- seglenh in each direction.  The orthogonal length is never allowed to be less than 16 pixels to preserve accuracy.
    dist=max([fix(fix(max(distim[input[n,0]-seglenh:input[n,0]+seglenh,input[n,1]-seglenh:input[n,1]+seglenh]))*2*1.25),16])

    ;The hypotenuse of our triangle, set up by our centerline length and our 
    hyp=sqrt((float(seglenh)^2)+(float(dist)^2))

    distdif=5.0
    pt1=intarr(2)
    pt2=intarr(2)
    begin_dist=10
    
    ;2.1) This loop finds the two correct endpoints of the centerline segment against which the orthogonal will be computed 
    for d=0,seglenh do begin
       ;Determines the centerline segment endpoint in the upstream direction
       if distance(input[n-d,0],input[n-d,1],input[n,0],input[n,1]) gt seglenh then begin
         ;Basically iterates downstream until a pixel with a euclidean distance of gt seglenh from the center pixel is found.  
         ;This takes care of the problem that not all pixels are 1 pixel length from one another in the centerline (ie some are diagonal and thus sqrt(2) distant)
         old_begin_dist=begin_dist
         begin_dist=distance(input[n-d,0],input[n-d,1],input[n,0],input[n,1])-seglenh
         if begin_dist lt old_begin_dist and begin_dist ge 0 then begin
          spx=input[n-d,0]
          spy=input[n-d,1]
         endif
       endif else begin
         spx=input[n-seglenh,0]
         spy=input[n-seglenh,1]
       endelse
       
       ;Determines the centerline segment endpoint in the downstream direction
       if distance(input[n+d,0],input[n+d,1],input[n,0],input[n,1]) gt seglenh then begin
          old_begin_dist=begin_dist
          begin_dist=distance(input[n+d,0],input[n+d,1],input[n,0],input[n,1])-seglenh
          if begin_dist lt old_begin_dist and begin_dist ge 0 then begin
           epx=input[n+d,0]
           epy=input[n+d,1]
          endif
       endif else begin
         epx=input[n+seglenh,0]
         epy=input[n+seglenh,1]
       endelse
    endfor
    
    cpx=long(spx+(epx-spx)/2)
    cpy=long(spy+(epy-spy)/2)
    if abs(spx-epx) ge abs(spy-epy) then divd=1
    if abs(spx-epx) lt abs(spy-epy) then divd=0
    orthd=dist
    orthd2=dist+1
    hyp=sqrt((float(seglenh)^2)+(float(orthd)^2))  ; D. Mayer changed hyp to return a floating point number

    ;2.2) The following segment of code determines the x/y coordinates of the orthogonal endpoints

    ;2.2.1) If the centerline vector is moving more in the x direction, then we split the image into two halves horizontally (parallel to the x axis)

    if divd eq 1 then begin
      ;2.2.1.1) Bottom Half of the Image
      diste=distfull[dfval/2-orthd2:dfval/2+orthd2,dfval/2-orthd2:dfval/2]
      distc=distfull[dfval/2-orthd2-(epx-cpx):dfval/2+orthd2-(epx-cpx),dfval/2-orthd2-(epy-cpy):dfval/2-(epy-cpy)]
      disteh=diste-hyp
      distcd=distc-orthd
      
      outpoint=where(abs(disteh) le 0.5 and abs(distcd) le 0.5)
      if outpoint[0] ne long(-1) then outvals=disteh(where(abs(disteh) le 0.5 and abs(distcd) le 0.5)) else outvals=0 ;Added by D.Mayer to match section 2.2.2 below
      finalpts=outpoint(where(outvals eq min(outvals)))
      finalpt=finalpts[0]
      array_size=size(disteh, /dimensions)
      arrayx=array_size[0]
      finx=cpx+long((finalpt mod arrayx)-arrayx/2)
      finy=cpy+long(finalpt/arrayx)-arrayx/2      
      pt1=[finx,finy]

      ;2.2.1.2) Top Half of the Image
      diste=distfull[dfval/2-orthd2:dfval/2+orthd2,dfval/2+1:dfval/2+1+orthd2]
      distc=distfull[dfval/2-orthd2-(epx-cpx):dfval/2+orthd2-(epx-cpx),dfval/2+1-(epy-cpy):dfval/2+1+orthd2-(epy-cpy)]      
      disteh=diste-hyp
      distcd=distc-orthd
      
      outpoint=where(abs(disteh) le 0.5 and abs(distcd) le 0.5)
      if outpoint[0] ne long(-1) then outvals=disteh(where(abs(disteh) le 0.5 and abs(distcd) le 0.5)) else outvals=0 ;Added by D.Mayer to match section 2.2.2 below
      finalpts=outpoint(where(outvals eq min(outvals)))
      finalpt=finalpts[0]
      array_size=size(disteh, /dimensions)
      arrayx=array_size[0]
      arrayy=array_size[1]
      finx=cpx+long((finalpt mod arrayx)-arrayx/2)
      finy=cpy+long(finalpt/arrayx)          
      pt2=[finx,finy]
    endif
    
    ;2.2.2) If the vector is moving more in the y direction, then we split the image into two halves vertically (parallel to the y axis)

    if divd eq 0 then begin
      ;2.2.2.1) Left half of the image
      diste=distfull[dfval/2-orthd2:dfval/2,dfval/2-orthd2:dfval/2+orthd2]
      distc=distfull[dfval/2-orthd2-(epx-cpx):dfval/2-(epx-cpx),dfval/2-orthd2-(epy-cpy):dfval/2+orthd2-(epy-cpy)]      
      disteh=diste-hyp
      distcd=distc-orthd
      
      outpoint=where(abs(disteh) le 0.5 and abs(distcd) le 0.5)
      if outpoint[0] ne long(-1) then outvals=disteh(where(abs(disteh) le 0.5 and abs(distcd) le 0.5)) else outvals=0
      finalpts=outpoint(where(outvals eq min(outvals)))
      finalpt=finalpts[0]
      array_size=size(disteh, /dimensions)
      arrayx=array_size[0]
      arrayy=array_size[1]
      finx=cpx+long(finalpt mod arrayx)-long(arrayx)
      finy=cpy+long(finalpt/arrayx)-long(arrayy/2)  
      pt1=[finx,finy]

      ;2.2.2.2) Right half of the image
      diste=distfull[dfval/2+1:dfval/2+1+orthd2,dfval/2-orthd2:dfval/2+orthd2]
      distc=distfull[dfval/2+1-(epx-cpx):dfval/2+1+orthd2-(epx-cpx),dfval/2-orthd2-(epy-cpy):dfval/2+orthd2-(epy-cpy)]      
      disteh=diste-hyp
      distcd=distc-orthd
      
      outpoint=where(abs(disteh) le 0.5 and abs(distcd) le 0.5)
      if outpoint[0] ne long(-1) then outvals=disteh(where(abs(disteh) le 0.5 and abs(distcd) le 0.5)) else outvals=0 ;Added by D.Mayer to match section 2.2.2.1
      finalpts=outpoint(where(outvals eq min(outvals)))
      finalpt=finalpts[0]
      array_size=size(disteh, /dimensions)
      arrayx=array_size[0]
      arrayy=array_size[1]
      finx=cpx+long(finalpt mod arrayx)
      finy=cpy+long(finalpt/arrayx)-long(arrayy/2)          
      pt2=[finx,finy]
 
    endif
    
    ;The following code only matters if you are computing average width across more than 1 centerline pixel (see the control function for more explanation)
    if (n MOD agval) eq agval/2 AND n ge agval AND bound(pt1[0],pt1[1],width,height) eq 1 and bound(pt2[0],pt2[1],width,height) eq 1 then begin
       orth_image[pt1[0],pt1[1]]=3
       orth_image[pt2[0],pt2[1]]=2
       plotptsx=[pt1[0],pt2[0]]
       plotptsy=[pt1[1],pt2[1]]
       orth_image[input[n,0],input[n,1]]=1
    endif
    
    ortharr[n,0:7]=[pt1[0],pt1[1],pt2[0],pt2[1],input[n,0],input[n,1],cpx,cpy]
    n=n+1
    
    ;if n mod 100 eq 0 then print, "Finished calcuating ", n, " of ", final-seglenh, " orthogonals"
endwhile

;;shows the center line along with the points used to compute the orthogonal
;openw, 1, FILEPATH('cline_plus_orthpoints_newtest2',ROOT_DIR='/Volumes/Data/files/Alaska/Yukon_width/')
;
;writeu, 1, orth_image
;
;close, 1

return, ortharr  ;Returns an array containing the x,y coordinates of the centerline pixels along with the endpoints of the orthogonals.  
                 ;This is the primary input for z_values_v3, below.  It is assigned to the variable inarr in z_values_v3.
end

;;;;;;;;;
;Subfunctions of function Return_orthogonal_v3 above
;;;;;;;;;

;This funciton calculates and returns the euclidean distance between two points.
function distance, ax,ay,bx,by
    dstance=sqrt(long(ax-bx)^2 + long(ay-by)^2)
    return, dstance
end

;This function determines whether a pixel lies on a boundary of the image.
function bound, a,b,width,height
  if a lt 0 OR a gt width-1 then return,0
  if b lt 0 OR b gt height-1 then return,0
  return, 1
end

;;;;;;;;;;;;;;;;;;;
;Begin Z_values section
;This section should be edited only under very unusual circumstances.
;;;;;;;;;;;;;;;;;;;

function z_values_v3, inim, inarr, agval, pixel_size, rivmask

;1)  Initializes a whole bunch of variables used further down
inarr_size=size(inarr, /Dimensions)
tot_n=inarr_size[0]
inim_size=size(inim, /Dimensions)
nx=inim_size[0]
ny=inim_size[1]
finalout=lonarr(tot_n,8)
finalimage=lonarr(nx,ny)

xout = lonarr(tot_n,tot_n/2)
yout = lonarr(tot_n,tot_n/2)
zout = fltarr(tot_n,tot_n/2)
rivzout=fltarr(tot_n,tot_n/2)

u=1

;2) The following for loop goes through each centerline pixel and extracts the line of pixels from the channel mask that falls along the
;   orthogonal to that pixel.  So what you end up with are three arrays (xout, yout, and zout) which contain, respectively, the x coordinate
;   y coordinate, and value (either 0 or 1) of each pixel in the centerline.  Note on attribution:  this loop is heavily modified from some
;   code originally by Doug Alsdorf.

for i=0, tot_n-1 do begin
    if (inarr[i,0] eq 0 AND inarr[i,1] eq 0) OR (inarr[i,2] eq 0 AND inarr[i,3] eq 0) AND i lt tot_n/2 then u=i
    if (inarr[i,0] gt 0 OR inarr[i,1] gt 0) AND (inarr[i,2] gt 0 OR inarr[i,3] gt 0) then begin
         dx=inarr[i,0]-inarr[i,2]
         dy=inarr[i,1]-inarr[i,3]
         n = abs(dx) > abs(dy)
         if abs(dx) gt abs(dy) then begin
            if inarr[i,0] ge inarr[i,2] then s=1 else s=-1
            sy = float( float(inarr[i,1]-inarr[i,3])/float(abs(dx)) )
         endif else begin
            if inarr[i,1] ge inarr[i,3] then sy=1 else sy=-1
            s = float( float(inarr[i,0]-inarr[i,2])/float(abs(dy)) )
         endelse
         xx = long(lindgen(n+1l)*s+inarr[i,2])
         yy = long(lindgen(n+1l)*sy+inarr[i,3])
         if min(xx) lt 0 or max(xx) ge nx or min(yy) lt 0 or max(yy) ge ny then begin
            wherevals=where(xx ge 0 and xx lt nx and yy ge 0 and yy lt ny)
            xx=xx(wherevals)
            yy=yy(wherevals)
         endif
         xysize=size(yy, /dimensions)
         xylen=xysize[0]
         zz = inim[long(yy)*nx + xx]
         rivzz = rivmask[long(yy)*nx + xx]
         
         xout[i,0:xylen-1]=xx & yout[i,0:xylen-1]=yy & zout[i,0:xylen-1]=zz & rivzout[i,0:xylen-1]=rivzz

     endif
endfor

;3) The following while loop goes through each pixel in the centerline and adds up the euclidean distance across each channel intersected by each orthogonal cross section.
;   What we get in the end is an array, finalout, which contains the x/y coordinates of the two orthogonal pixels and the centerline pixel (positions 0-5 in the second dimension
;   of finalout), the total flow width in whatever units the resolution of the images is provided in (position 6), and the total number of channels crossed (position 7).

while u lt tot_n-agval-1 do begin
    counter=0
    s=0
    switch_var=zout[u,0]
    sv2=rivzout[u,0]
    svc=0
    start_pixel=lonarr(2)
    switch_count=0
    
    for t=0,s do begin
       counter=0
       tot_dist=double(0.0)
       start_pixel=[xout[u+t,0],yout[u+t,0]]
       rivlabel=fltarr(tot_n/2)
       rivdist=fltarr(tot_n/2)
       for f=0,tot_n/2-1 do begin
        if rivzout[u+t,f] eq sv2 then begin
          sv2=abs(sv2-1)
          svc++
        endif
        if rivzout[u+t,f] eq 1 then begin
          rivlabel[f]=svc
          rivdist[f]=distance(xout[u+t,f],yout[u+t,f],inarr[u+t,6],inarr[u+t,7])
          if xout[u+t,f] eq inarr[u+t,6] and yout[u+t,f] eq inarr[u+t,7] then rivdist[f]=-1
        endif
       endfor
       if svc gt 3 then begin
        rightlabel=rivlabel(where(rivdist eq min(rivdist(where(rivdist ne 0)))))
        rightloc=where(rivdist eq min(rivdist(where(rivdist ne 0))))
        ;if max(rivlabel) ne 0 then print, max(rivlabel), min(rivdist(where(rivdist ne 0))), max(rightlabel)
        zvals=reform(zout[u+t,*])
        zvalssafe=zvals
        rivzvals=reform(rivzout[u+t,*])
        if rivzvals(rightloc[0]) eq 1 then zvals(where(rivlabel ne rightlabel[0]))=0 else print, "Error in correcting width.  Value for point ", inarr[u+t,4], inarr[u+t,5], " is unreliable."
        zout[u+t,*]=zvals
       endif else zvalssafe=reform(zout[u+t,*])
                  
       for z=0,tot_n/2-1 do begin
         if zout[u+t,z] eq switch_var then begin
          switch_var=abs(switch_var-1)
          switch_count++
          if switch_var eq 1 AND xout[u+t,z] gt 0 then begin
              tot_dist=tot_dist+distance(start_pixel[0],start_pixel[1],xout[u+t,z],yout[u+t,z])             
          endif
          start_pixel[0]=xout[u+t,z]
          start_pixel[1]=yout[u+t,z]         
         endif        
       endfor
       zout[u+t,*]=zvalssafe
       n_channel=switch_count/2
       finalout[u+t,0]=inarr[u+t,0]
       finalout[u+t,1]=inarr[u+t,1]
       finalout[u+t,2]=inarr[u+t,2]
       finalout[u+t,3]=inarr[u+t,3]
       finalout[u+t,4]=inarr[u+t,4]
       finalout[u+t,5]=inarr[u+t,5]
       finalout[u+t,6]=tot_dist*pixel_size
       finalout[u+t,7]=n_channel
    endfor

    u=u+s+1
    if u ge tot_n then u=tot_n-1

endwhile

;;this image shows the boundaries of the cross-sectional windows used to compute width
;openw, 6, FILEPATH('image_crossections_ohio_40',ROOT_DIR='/Files/SWOT/Rivwidth/')
;
;writeu, 6, finalimage
;
;close, 6

return, finalout

end


function read_inputs,inputparams

;Function to read the input data from the parameter file
;  by Mike Durand, April 29, 2008

;1) Open the input file
openr,1, inputparams

;2) Read the input file
while eof(1) eq 0 do begin
  ;2.1) Declare buffer as an empty string 
  buffer=''
  ;2.2) Read one line of input from the file
  readf,1,buffer
  ;2.3) Split the line into the name/value pair
  args=strsplit(buffer,/extract)
  arg_name=args(0)
  arg_value=args(1)
  
  ;2.4) Query each of the names, and copy values to correct varaibles
  ;2.4.1) Image details
  if(strmatch(arg_name,'width') eq 1) then width=long(arg_value)
  if(strmatch(arg_name,'height') eq 1) then height=long(arg_value)
  if(strmatch(arg_name,'resolution') eq 1) then resolution=uint(arg_value)
  if(strmatch(arg_name,'braidnarrowflag') eq 1) then braidnarrowflag=uint(arg_value)
  ;2.4.2) Input file name - river classification
  if(strmatch(arg_name,'water_class') eq 1) then water_class_name=arg_value
  if(strmatch(arg_name,'riv_class') eq 1) then  riv_class_name=arg_value
  if(strmatch(arg_name,'chan_class') eq 1) then  chan_class_name=arg_value   
  ;2.4.3) Output file names
  if(strmatch(arg_name,'dist_image') eq 1) then dist_img_name=arg_value
  if(strmatch(arg_name,'cl_image') eq 1) then cl_img_name=arg_value
  if(strmatch(arg_name,'cl1_image') eq 1) then cl_img_name1=arg_value
  if(strmatch(arg_name,'w_image') eq 1) then w_img_name=arg_value
  if(strmatch(arg_name,'w_csv') eq 1) then w_csv_name=arg_value
  ;2.4.4) Switches to control program flow    
  if(strmatch(arg_name,'calc_masks') eq 1) then dist_sw=uint(arg_value)
  if(strmatch(arg_name,'calc_centerline') eq 1) then cl_sw=uint(arg_value)
  if(strmatch(arg_name,'calc_width') eq 1) then w_sw=uint(arg_value)
  ;2.4.5) Start and stop image pixels from ENVI
  if(strmatch(arg_name,'spx') eq 1) then spx=uint(arg_value)
  if(strmatch(arg_name,'spy') eq 1) then spy=uint(arg_value)
  if(strmatch(arg_name,'epx') eq 1) then epx=uint(arg_value)
  if(strmatch(arg_name,'epy') eq 1) then epy=uint(arg_value)  
    
endwhile

;3) Close input file
close, 1

;4) Copy input variables to a structure, and return it
rundata={w:width,h:height,r:resolution,b:braidnarrowflag,$
          f0:water_class_name,f1:riv_class_name,f2:chan_class_name,f3:dist_img_name,f4:cl_img_name,f5:cl_img_name1,$
          f6:w_img_name,f7:w_csv_name,s1:dist_sw,s2:cl_sw,s3:w_sw,spx:spx,spy:spy,epx:epx,epy:epy}
return, rundata

end
